﻿namespace Sample
{
	using System;
	using System.ComponentModel;
	using System.Windows;
	using System.Windows.Forms;
	using MessageBox = System.Windows.MessageBox;

	using MoreLinq;

	using Ecng.Collections;
	using Ecng.Common;
	using Ecng.Xaml;

	using StockSharp.BusinessEntities;
	using StockSharp.Quik;
    using StockSharp.Algo;
    using System.Collections.Generic;


	public partial class MainWindow
	{
		public QuikTrader Trader;

		private readonly SecuritiesWindow _securitiesWindow = new SecuritiesWindow();
		private readonly TradesWindow _tradesWindow = new TradesWindow();
		private readonly MyTradesWindow _myTradesWindow = new MyTradesWindow();
		private readonly OrdersWindow _ordersWindow = new OrdersWindow();
		private readonly PortfoliosWindow _portfoliosWindow = new PortfoliosWindow();
		private readonly StopOrderWindow _stopOrderWindow = new StopOrderWindow();
        private readonly object _locker = new object(); //локер
        private  Order _order;
        private Security _security;
        private Portfolio _portfolio;

		public MainWindow()
		{
			InitializeComponent();
			MainWindow.Instance = this;

			_ordersWindow.MakeHideable();
			_myTradesWindow.MakeHideable();
			_tradesWindow.MakeHideable();
			_securitiesWindow.MakeHideable();
			_stopOrderWindow.MakeHideable();
			_portfoliosWindow.MakeHideable();

			// попробовать сразу найти месторасположение Quik по запущенному процессу
			Path.Text = QuikTerminal.GetDefaultPath();
		}

		protected override void OnClosing(CancelEventArgs e)
		{
			_ordersWindow.DeleteHideable();
			_myTradesWindow.DeleteHideable();
			_tradesWindow.DeleteHideable();
			_securitiesWindow.DeleteHideable();
			_stopOrderWindow.DeleteHideable();
			_portfoliosWindow.DeleteHideable();
			
			_securitiesWindow.Close();
			_tradesWindow.Close();
			_myTradesWindow.Close();
			_stopOrderWindow.Close();
			_ordersWindow.Close();
			_portfoliosWindow.Close();

			if (Trader != null)
			{
				if (_isDdeStarted)
					StopDde();

				Trader.Dispose();
			}

			base.OnClosing(e);
		}

		public static MainWindow Instance { get; private set; }

		private void FindPathClick(object sender, RoutedEventArgs e)
		{
			var dlg = new FolderBrowserDialog();

			if (!Path.Text.IsEmpty())
				dlg.SelectedPath = Path.Text;

			if (dlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
			{
				Path.Text = dlg.SelectedPath;
			}
		}

		private bool _isConnected;

		private void ConnectClick(object sender, RoutedEventArgs e)
		{
			if (!_isConnected)
			{
				if (Path.Text.IsEmpty())
					MessageBox.Show(this, "Путь к Quik не выбран");
				else
				{
					if (Trader == null)
					{
						// создаем подключение
						Trader = new QuikTrader(Path.Text);

                        Trader.NewSecurities += Securities =>
                        {
                            this.GuiAsync(() => SecComboBox.ItemsSource = Trader.Securities);
                        };
                        Trader.NewPortfolios += Portfolios =>
                        {
                            this.GuiAsync(() =>  PortfolioComboBox.ItemsSource = Trader.Portfolios);
                        };

						// возводим флаг, что соединение установлено
						_isConnected = true;

						// переподключение будет работать только во время работы биржи РТС
						// (чтобы отключить переподключение когда торгов нет штатно, например, ночью)
						Trader.ReConnectionSettings.WorkingTime = ExchangeBoard.Forts.WorkingTime;

						// подписываемся на событие об успешном восстановлении соединения
						Trader.ReConnectionSettings.ConnectionSettings.Restored += () => this.GuiAsync(() => MessageBox.Show(this, "Соединение восстановлено"));

						// подписываемся на событие разрыва соединения

						Trader.ConnectionError += error => this.GuiAsync(() => MessageBox.Show(this, error.ToString()));

						Trader.NewSecurities += securities => this.GuiAsync(() => _securitiesWindow.SecurityPicker.Securities.AddRange(securities));
						Trader.NewMyTrades += trades => this.GuiAsync(() => _myTradesWindow.TradeGrid.Trades.AddRange(trades));
						Trader.NewTrades += trades => this.GuiAsync(() => _tradesWindow.TradeGrid.Trades.AddRange(trades));
						Trader.NewOrders += orders => this.GuiAsync(() => _ordersWindow.OrderGrid.Orders.AddRange(orders));
						Trader.NewStopOrders += orders => this.GuiAsync(() => _stopOrderWindow.Orders.AddRange(orders));
						Trader.OrdersRegisterFailed += fails => fails.ForEach(fail => this.GuiAsync(() => MessageBox.Show(this, fail.Error.Message, "Ошибка регистрации заявки")));
						Trader.OrdersCancelFailed += fails => fails.ForEach(fail => this.GuiAsync(() => MessageBox.Show(this, fail.Error.Message, "Ошибка отмены заявки")));
						Trader.NewPortfolios += portfolios => this.GuiAsync(() => _portfoliosWindow.PortfolioGrid.Portfolios.AddRange(portfolios));
						Trader.NewPositions += positions => this.GuiAsync(() => _portfoliosWindow.PortfolioGrid.Positions.AddRange(positions));
						Trader.ProcessDataError += ex => System.Diagnostics.Debug.WriteLine(ex);
						Trader.Connected += () => this.GuiAsync(() => ExportDde.IsEnabled = true);

						ShowSecurities.IsEnabled = ShowTrades.IsEnabled =
						ShowMyTrades.IsEnabled = ShowOrders.IsEnabled =
						ShowPortfolios.IsEnabled = ShowStopOrders.IsEnabled = true;
					}

					Trader.Connect();

					_isConnected = true;
					ConnectBtn.Content = "Отключиться";
				}
			}
			else
			{
				Trader.Disconnect();

				_isConnected = false;
				ConnectBtn.Content = "Подключиться";
			}
		}

		private void ShowSecuritiesClick(object sender, RoutedEventArgs e)
		{
			ShowOrHide(_securitiesWindow);
		}

		private void ShowTradesClick(object sender, RoutedEventArgs e)
		{
			ShowOrHide(_tradesWindow);
		}

		private void ShowMyTradesClick(object sender, RoutedEventArgs e)
		{
			ShowOrHide(_myTradesWindow);
		}

		private void ShowOrdersClick(object sender, RoutedEventArgs e)
		{
			ShowOrHide(_ordersWindow);
		}

		private void ShowPortfoliosClick(object sender, RoutedEventArgs e)
		{
			ShowOrHide(_portfoliosWindow);
		}

		private void ShowStopOrdersClick(object sender, RoutedEventArgs e)
		{
			ShowOrHide(_stopOrderWindow);
		}

		private static void ShowOrHide(Window window)
		{
			if (window == null)
				throw new ArgumentNullException("window");

			if (window.Visibility == Visibility.Visible)
				window.Hide();
			else
				window.Show();
		}

		private bool _isDdeStarted;

		private void StartDde()
		{
			Trader.StartExport();
			_isDdeStarted = true;
		}

		private void StopDde()
		{
			Trader.StopExport();
			_isDdeStarted = false;
		}

		private void ExportDdeClick(object sender, RoutedEventArgs e)
		{
			if (_isDdeStarted)
				StopDde();
			else
				StartDde();
		}

        private void StartButton_Click(object sender, RoutedEventArgs e)
        {
            _security = (Security)SecComboBox.SelectedValue;
            _portfolio = (Portfolio)PortfolioComboBox.SelectedValue;
            Trader.RegisterMarketDepth(_security);
            Trader.MarketDepthsChanged += Algoritm;

        }

        private void Algoritm(IEnumerable<MarketDepth> depths) // обновляем все данные
        //        private void Algoritm(TimeSpan t) // обновляем все данные
        {
            lock (_locker)
            {
                if (_order == null && _security.BestAsk != null)
                {
                    _order = new Order()
                    {
                        Volume = 1,
                        Price = _security.BestAsk.Price + 10,
                        Security = _security,
                        Portfolio = _portfolio,
                        Direction = OrderDirections.Sell

                    };
                    Trader.RegisterOrder(_order);
                }

                else if (_order != null && _order.State == StockSharp.Messages.OrderStates.Active && _order.Price != _security.BestAsk.Price + 10)
                {
                    var OldOrder = _order;
                    _order = null;
                    var NewReRegOrder = OldOrder.ReRegisterClone(_security.BestAsk.Price + 5);
                    _order = NewReRegOrder;
                    Trader.ReRegisterOrder(OldOrder, NewReRegOrder);
                };
            }
        }

	}
}
